@@ -94,7 +94,7 @@
 #'  with the number of the model term attached.
 #' @param debug if \code{TRUE} a breakpoint is set at the beginning of the posterior
 #'  draw function associated with this model component. Mainly intended for developers.
-#' @return an object with precomputed quantities and functions for sampling from
+#' @returns An object with precomputed quantities and functions for sampling from
 #'  prior or conditional posterior distributions for this model component. Intended
 #'  for internal use by other package functions.
 reg <- function(formula = ~ 1, remove.redundant=FALSE, sparse=NULL, X=NULL,
@@ -107,11 +107,14 @@ reg <- function(formula = ~ 1, remove.redundant=FALSE, sparse=NULL, X=NULL,
   if (name == "") stop("missing model component name")
 
   if (e$family$family == "gamma") {
-    modus <- "gamma"    # model for log(mean) of gamma
-  } else if (name %in% names(e$Vmod)) {
-    modus <- "var"      # model for log(var) of gaussian
+    modus <- "gamma"       # model for log(mean) of gamma
+  } else if (any(name == names(e$Vmod))) {
+    if (e$family$family == "gaussian_gamma")
+      modus <- "vargamma"  # model for log(var) of gaussian and log(mean) of gamma
+    else
+      modus <- "var"       # model for log(var) of gaussian
   } else {
-    modus <- "regular"  # model for mean of gaussian/binomial/...
+    modus <- "regular"     # model for mean of gaussian/binomial/...
   }
 
   if (is.null(X)) {
@@ -119,7 +122,7 @@ reg <- function(formula = ~ 1, remove.redundant=FALSE, sparse=NULL, X=NULL,
       edat <- new.env(parent = .GlobalEnv)
       X <- NULL
       for (k in seq_len(e$Km1)) {
-        edat$cat_ <- factor(rep.int(e$cats[k], e$n0), levels=e$cats[-length(e$cats)])
+        edat$cat_ <- factor(rep.int(e$cats[k], e[["n0"]]), levels=e$cats[-length(e$cats)])
         X <- rbind(X, model_matrix(formula, data=e$data, sparse=sparse, enclos=edat))
       }
       rm(edat)
@@ -130,7 +133,7 @@ reg <- function(formula = ~ 1, remove.redundant=FALSE, sparse=NULL, X=NULL,
   }
   X <- economizeMatrix(X, sparse=sparse, strip.names=FALSE, check=TRUE)
 
-  if (nrow(X) != e$n) stop("design matrix with incompatible number of rows")
+  if (nrow(X) != e[["n"]]) stop("design matrix with incompatible number of rows")
   e$coef.names[[name]] <- colnames(X)
   X <- unname(X)
   q <- ncol(X)
@@ -201,14 +204,18 @@ reg <- function(formula = ~ 1, remove.redundant=FALSE, sparse=NULL, X=NULL,
     if (!zero.mean) stop("weights cannot be computed if some coefficients have non-zero prior means")
   }
 
-  if (modus == "var") {
-    compute_Qfactor <- function(p) exp(X %m*v% (- p[[name]]))
+  if (modus == "var" || modus == "vargamma") {
+    if (is_ind_matrix(X) && q < e[["n"]])
+      compute_Qfactor <- function(p) X %m*v% exp(-p[[name]])
+    else
+      compute_Qfactor <- function(p) exp(X %m*v% (-p[[name]]))
   }
   linpred <- function(p) X %m*v% p[[name]]
 
   # for regression components assume that categorical variables in newdata
   #   carry the exact same levels as those in the training data
   make_predict <- function(newdata=NULL, Xnew, verbose=TRUE) {
+    if (modus == "vargamma") stop("prediction for 'gaussian_gamma' family not supported")
     if (is.null(newdata)) {
       if (missing(Xnew)) stop("one of 'newdata' and 'Xnew' should be supplied")
       if (is.null(colnames(Xnew))) {
@@ -233,8 +240,13 @@ reg <- function(formula = ~ 1, remove.redundant=FALSE, sparse=NULL, X=NULL,
       Xnew <- Xnew[, e$coef.names[[name]], drop=FALSE]
     } else {
       if (ncol(Xnew) != q) stop("'newdata' yields ", ncol(Xnew), " predictor column(s) for model term '", name, "' versus ", q, " originally")
-      if (!is.null(colnames(Xnew)) && !identical(colnames(Xnew), e$coef.names[[name]]))
+      if (!is.null(colnames(Xnew)) && !identical(colnames(Xnew), e$coef.names[[name]])) {
+        if (!all(e$coef.names[[name]] %in% colnames(Xnew))) {
+          stop("the following model matrix columns could not be derived from Xnew: ",
+            paste(setdiff(e$coef.names[[name]], colnames(Xnew)), collapse=", "))
+        }
         Xnew <- Xnew[, e$coef.names[[name]], drop=FALSE]
+      }
     }
     Xnew <- economizeMatrix(Xnew, sparse=sparse, strip.names=TRUE, check=TRUE)
     rm(newdata, verbose)
@@ -244,9 +256,13 @@ reg <- function(formula = ~ 1, remove.redundant=FALSE, sparse=NULL, X=NULL,
   if (!in_block && !e$prior.only && prior$type != "fixed") {
 
     draw <- if (debug) function(p) {browser()} else function(p) {}
-    if (modus == "var") {
-      if (!e$single.V.block)
-        draw <- add(draw, bquote(p[["Q_"]] <- p[["Q_"]] * exp(X %m*v% p[[.(name)]])))
+    if (modus == "var" || modus == "vargamma") {
+      if (!e$single.V.block) {
+        if (is_ind_matrix(X) && q < e[["n"]])
+          draw <- add(draw, bquote(p[["Q_"]] <- p[["Q_"]] * (X %m*v% exp(p[[.(name)]]))))
+        else
+          draw <- add(draw, bquote(p[["Q_"]] <- p[["Q_"]] * exp(X %m*v% p[[.(name)]])))
+      }
     } else {
       if (e$single.block) {
         if (e$use.offset || e$e.is.res)
@@ -259,49 +275,68 @@ reg <- function(formula = ~ 1, remove.redundant=FALSE, sparse=NULL, X=NULL,
       }
     }
     if (e$single.block && e$family$link != "probit" && modus == "regular" &&
-        (!e$modeled.Q || (e$family$family != "gaussian" && !e$use.offset && !e$modeled.r))) {
+        (!e$modeled.Q || (all(e$family$family != c("gaussian", "gaussian_gamma")) && !e$use.offset && !e$modeled.r))) {
       # single regression component, no variance modeling, Xy fixed
       if (e$family$family == "gaussian")
         Xy <- crossprod_mv(X, e$Q0 %m*v% e$y_eff()) + Q0b0
       else
         Xy <- crossprod_mv(X, e$Q_e(e$y_eff())) + Q0b0
-    } else {  # Xy updated in each iteration
-      if (modus == "regular") {
-        if (all(Q0b0 == 0))
-          draw <- add(draw, quote(Xy <- crossprod_mv(X, e$Q_e(p))))
+    } else if (modus == "regular") {
+      # Xy updated in each iteration
+      if (all(Q0b0 == 0))
+        draw <- add(draw, quote(Xy <- crossprod_mv(X, e$Q_e(p))))
+      else
+        draw <- add(draw, quote(Xy <- crossprod_mv(X, e$Q_e(p)) + Q0b0))
+    } else {  # modus gamma, var, vargamma
+      if (modus == "var" || modus == "vargamma") {  # variance modelling
+        if (e$single.V.block)
+          draw <- add(draw, bquote(vkappa <- .(if (e$sigma.fixed) 0.5 else quote(0.5/p[["sigma_"]]^2)) * p[["e_"]]^2))
         else
-          draw <- add(draw, quote(Xy <- crossprod_mv(X, e$Q_e(p)) + Q0b0))
-      } else {
-        if (modus == "gamma") {
-          if (e$family$alpha.fixed) {
-            alpha <- e$family$get_shape()
-            if (e$single.block && !e$use.offset)
-              kappa <- alpha * e$y
-            else {
-              kappa0 <- alpha * e$y
-              draw <- add(draw, quote(kappa <- kappa0 * exp(-p[["e_"]])))
-            }
-          } else {
-            draw <- add(draw, quote(alpha <- e$family$get_shape(p)))
-            if (e$single.block && !e$use.offset)
-              draw <- add(draw, quote(kappa <- alpha * e[["y"]]))
-            else
-              draw <- add(draw, quote(kappa <- alpha * e[["y"]] * exp(-p[["e_"]])))
+          draw <- add(draw, bquote(vkappa <- .(if (e$sigma.fixed) 0.5 else quote(0.5/p[["sigma_"]]^2)) * p[["e_"]]^2 * p[["Q_"]]))
+        draw <- add(draw, bquote(Hz <- crossprod_mv(X, rMLiG(.(e[["n"]]), 0.5, vkappa))))
+      }
+      if (modus == "gamma") {
+        if (e$family$alpha.fixed) {
+          alpha <- e$family$get_shape()
+          if (e$single.block && !e$use.offset)
+            kappa <- alpha * e[["y"]]
+          else {
+            kappa0 <- alpha * e[["y"]]
+            draw <- add(draw, quote(kappa <- kappa0 * exp(-p[["e_"]])))
           }
-        } else {  # variance modelling
-          alpha <- 0.5
+        } else {
+          draw <- add(draw, quote(alpha <- e$family$get_shape(p)))
+          if (e$single.block && !e$use.offset) {
+            draw <- add(draw, quote(kappa <- alpha * e[["y"]]))
+          } else {
+            draw <- add(draw, quote(kappa <- alpha * e[["y"]] * exp(-p[["e_"]])))
+          }
+        }
+        draw <- add(draw, bquote(Hz <- crossprod_mv(X, rMLiG(.(e[["n"]]), alpha, kappa))))
+      } else if (modus == "vargamma") {
+        if (e$family$alpha.fixed) {
+          alpha <- e$family$get_shape()
           if (e$single.V.block)
-            draw <- add(draw, bquote(kappa <- .(if (e$sigma.fixed) 0.5 else quote(0.5/p[["sigma_"]]^2)) * p[["e_"]]^2))
-          else
-            draw <- add(draw, bquote(kappa <- .(if (e$sigma.fixed) 0.5 else quote(0.5/p[["sigma_"]]^2)) * p[["e_"]]^2 * p[["Q_"]]))
-        }
-        draw <- add(draw, bquote(Hz <- crossprod_mv(X, rMLiG(.(e$n), alpha, kappa))))
-        if (informative.prior) {
-          if (zero.mean)
-            draw <- add(draw, bquote(Hz <- Hz + sqrt(prior$precision/prior$a) * rMLiG(.(q), prior$a, prior$a)))
-          else
-            draw <- add(draw, bquote(Hz <- Hz + sqrt(prior$precision/prior$a) * rMLiG(.(q), prior$a, prior$a * exp(sqrt(prior$precision/prior$a) * prior$mean))))
+            kappa <- alpha * e$family[["sigmasq"]]
+          else {
+            kappa0 <- alpha * e$family[["sigmasq"]]
+            draw <- add(draw, quote(kappa <- kappa0 * p[["Q_"]]))
+          }
+        } else {
+          draw <- add(draw, quote(alpha <- e$family$get_shape(p)))
+          if (e$single.V.block) {
+            draw <- add(draw, quote(kappa <- alpha * e$family[["sigmasq"]]))
+          } else {
+            draw <- add(draw, quote(kappa <- alpha * e$family[["sigmasq"]] * p[["Q_"]]))
+          }
         }
+        draw <- add(draw, bquote(Hz <- Hz + crossprod_mv(X, rMLiG(.(e[["n"]]), alpha, kappa))))
+      }
+      if (informative.prior) {
+        if (zero.mean)
+          draw <- add(draw, bquote(Hz <- Hz + sqrt(prior$precision/prior$a) * rMLiG(.(q), prior$a, prior$a)))
+        else
+          draw <- add(draw, bquote(Hz <- Hz + sqrt(prior$precision/prior$a) * rMLiG(.(q), prior$a, prior$a * exp(sqrt(prior$precision/prior$a) * prior$mean))))
       }
     }
 
@@ -312,16 +347,16 @@ reg <- function(formula = ~ 1, remove.redundant=FALSE, sparse=NULL, X=NULL,
         draw <- add(draw, quote(XX <- crossprod_sym(X, p[["Q_"]])))
       if (informative.prior) {
         # TODO instead of runif(n, ...) account for the structure in Qmod; lambda may not vary per unit!
-        mat_sum <- make_mat_sum(M0=Q0, M1=crossprod_sym(X, crossprod_sym(Diagonal(x=runif(e$n, 0.9, 1.1)), e$Q0)))
+        mat_sum <- make_mat_sum(M0=Q0, M1=crossprod_sym(X, crossprod_sym(Diagonal(x=runif(e[["n"]], 0.9, 1.1)), e$Q0)))
         MVNsampler <- create_TMVN_sampler(
-          Q=mat_sum(crossprod_sym(X, crossprod_sym(Diagonal(x=runif(e$n, 0.9, 1.1)), e$Q0))),
+          Q=mat_sum(crossprod_sym(X, crossprod_sym(Diagonal(x=runif(e[["n"]], 0.9, 1.1)), e$Q0))),
           update.Q=TRUE, name=name, R=R, r=r, S=S, s=s, lower=lower, upper=upper,
           chol.control=e$control$chol.control
         )
         draw <- add(draw, bquote(p <- MVNsampler$draw(p, .(if (e$sigma.fixed) 1 else quote(p[["sigma_"]])), Q=mat_sum(XX), Xy=Xy)))
       } else {
         MVNsampler <- create_TMVN_sampler(
-          Q=crossprod_sym(X, crossprod_sym(Diagonal(x=runif(e$n, 0.9, 1.1)), e$Q0)),
+          Q=crossprod_sym(X, crossprod_sym(Diagonal(x=runif(e[["n"]], 0.9, 1.1)), e$Q0)),
           update.Q=TRUE, name=name, R=R, r=r, S=S, s=s, lower=lower, upper=upper,
           chol.control=e$control$chol.control
         )
@@ -347,15 +382,25 @@ reg <- function(formula = ~ 1, remove.redundant=FALSE, sparse=NULL, X=NULL,
           draw <- add(draw, bquote(p[[.(name)]] <- MVNsampler$draw(p, .(if (e$sigma.fixed) 1 else quote(p[["sigma_"]])), Xy=Xy)[[.(name)]]))
         }
       } else {
-        cholHH <- build_chol(economizeMatrix(crossprod_sym(X, CdiagU(e$n)) + (prior$precision/prior$a) * CdiagU(q), symmetric=TRUE))
+        if (modus == "vargamma")
+          cholHH <- build_chol(economizeMatrix(2 * crossprod_sym(X, CdiagU(e[["n"]])) + (prior$precision/prior$a) * CdiagU(q), symmetric=TRUE))
+        else
+          cholHH <- build_chol(economizeMatrix(crossprod_sym(X, CdiagU(e[["n"]])) + (prior$precision/prior$a) * CdiagU(q), symmetric=TRUE))
         draw <- add(draw, bquote(p[[.(name)]] <- cholHH$solve(Hz)))
       }
     }
-    if (modus == "var") {
-      if (e$single.V.block)
-        draw <- add(draw, bquote(p[["Q_"]] <- exp(X %m*v% (-p[[.(name)]]))))
-      else
-        draw <- add(draw, bquote(p[["Q_"]] <- p[["Q_"]] * exp(X %m*v% (-p[[.(name)]]))))
+    if (modus == "var" || modus == "vargamma") {
+      if (e$single.V.block) {
+        if (is_ind_matrix(X) && q < e[["n"]])
+          draw <- add(draw, bquote(p[["Q_"]] <- X %m*v% exp(-p[[.(name)]])))
+        else
+          draw <- add(draw, bquote(p[["Q_"]] <- exp(X %m*v% (-p[[.(name)]]))))
+      } else {
+        if (is_ind_matrix(X) && q < e[["n"]])
+          draw <- add(draw, bquote(p[["Q_"]] <- p[["Q_"]] * (X %m*v% exp(-p[[.(name)]]))))
+        else
+          draw <- add(draw, bquote(p[["Q_"]] <- p[["Q_"]] * exp(X %m*v% (-p[[.(name)]]))))
+      }
     } else {
       if (e$e.is.res)
         draw <- add(draw, bquote(mv_update(p[["e_"]], plus=FALSE, X, p[[.(name)]])))
@@ -372,9 +417,10 @@ reg <- function(formula = ~ 1, remove.redundant=FALSE, sparse=NULL, X=NULL,
       start <- function(p) MVNsampler$start(p)
     } else {
       start <- function(p) {
-        if (is.null(p[[name]]))
-          p[[name]] <- rnorm(q)
-        else {
+        if (is.null(p[[name]])) {
+          # account for the scale of covariates to avoid over/underflow of exp(linpred)
+          p[[name]] <- rnorm(q) / colwise_maxabs(X)
+        } else {
           p[[name]] <- as.numeric(p[[name]])
           if (length(p[[name]]) != q) stop("start value for '", name, "' has wrong length")
         }
@@ -403,7 +449,20 @@ reg <- function(formula = ~ 1, remove.redundant=FALSE, sparse=NULL, X=NULL,
     }
   }
 
-  if (!in_block || is.null(S)) rm(R, r, S, s, lower, upper)
+  if (in_block && (!is.null(e$control$CG) || e$control$expanded.cMVN.sampler)) {
+    # TODO account for b0
+    if (informative.prior) {
+      cholQV <- build_chol(Q0, control=e$control$chol.control)
+      drawMVNvarQ <- function(p) {
+        y2 <- Crnorm(q)
+        cholQV$Ltimes(y2, transpose=FALSE)
+      }
+    } else {
+      drawMVNvarQ <- function(p) numeric(q)
+    }
+  }
+
+  if (!in_block) rm(R, r, S, s, lower, upper)
 
   environment()
 }
